
(define (domain sokoban)
  (:requirements :strips :typing)
  (:types player stone - occupant
	  xpos ypos - pos
	  occupant pos - object)
  (:predicates (clear ?x - xpos ?y - ypos)
	       (at ?o - occupant ?x - xpos ?y - ypos)
	       (is-goal ?x - xpos ?y - ypos)
	       (at-goal ?s - stone)
	       (inc ?p ?pp - pos)
	       (dec ?p ?pp - pos))

  (:action move-up
    :parameters (?p - player ?x - xpos ?y0 ?y1 - ypos)
    ;; :assoc "u"
    :precondition (and (at ?p ?x ?y0)
		       (clear ?x ?y1)
		       (inc ?y0 ?y1))
    :effect (and (not (at ?p ?x ?y0))
		 (at ?p ?x ?y1)
		 (not (clear ?x ?y1))
		 (clear ?x ?y0))
    )

  (:action move-down
    :parameters (?p - player ?x - xpos ?y0 ?y1 - ypos)
    ;; :assoc "d"
    :precondition (and (at ?p ?x ?y0)
		       (clear ?x ?y1)
		       (dec ?y0 ?y1))
    :effect (and (not (at ?p ?x ?y0))
		 (at ?p ?x ?y1)
		 (not (clear ?x ?y1))
		 (clear ?x ?y0))
    )

  (:action move-right
    :parameters (?p - player ?x0 ?x1 - xpos ?y - ypos)
    ;; :assoc "r"
    :precondition (and (at ?p ?x0 ?y)
		       (clear ?x1 ?y)
		       (inc ?x0 ?x1))
    :effect (and (not (at ?p ?x0 ?y))
		 (at ?p ?x1 ?y)
		 (not (clear ?x1 ?y))
		 (clear ?x0 ?y))
    )

  (:action move-left
    :parameters (?p - player ?x0 ?x1 - xpos ?y - ypos)
    ;; :assoc "l"
    :precondition (and (at ?p ?x0 ?y)
		       (clear ?x1 ?y)
		       (dec ?x0 ?x1))
    :effect (and (not (at ?p ?x0 ?y))
		 (at ?p ?x1 ?y)
		 (not (clear ?x1 ?y))
		 (clear ?x0 ?y))
    )

  ;; push to non-goal

  (:action push-up
    :parameters (?p - player ?s - stone ?x - xpos ?y0 ?y1 ?y2 - ypos)
    ;; :assoc "U"
    :precondition (and (at ?p ?x ?y0)
		       (at ?s ?x ?y1)
		       (clear ?x ?y2)
		       (inc ?y0 ?y1)
		       (inc ?y1 ?y2)
		       (not (is-goal ?x ?y2)))
    :effect (and (not (at ?p ?x ?y0))
		 (not (at ?s ?x ?y1))
		 (not (clear ?x ?y2))
		 (not (at-goal ?s))
		 (at ?p ?x ?y1)
		 (at ?s ?x ?y2)
		 (clear ?x ?y0))
    )

  (:action push-down
    :parameters (?p - player ?s - stone ?x - xpos ?y0 ?y1 ?y2 - ypos)
    ;; :assoc "D"
    :precondition (and (at ?p ?x ?y0)
		       (at ?s ?x ?y1)
		       (clear ?x ?y2)
		       (dec ?y0 ?y1)
		       (dec ?y1 ?y2)
		       (not (is-goal ?x ?y2)))
    :effect (and (not (at ?p ?x ?y0))
		 (not (at ?s ?x ?y1))
		 (not (clear ?x ?y2))
		 (not (at-goal ?s))
		 (at ?p ?x ?y1)
		 (at ?s ?x ?y2)
		 (clear ?x ?y0))
    )

  (:action push-right
    :parameters (?p - player ?s - stone ?x0 ?x1 ?x2 - xpos ?y - ypos)
    ;; :assoc "R"
    :precondition (and (at ?p ?x0 ?y)
		       (at ?s ?x1 ?y)
		       (clear ?x2 ?y)
		       (inc ?x0 ?x1)
		       (inc ?x1 ?x2)
		       (not (is-goal ?x2 ?y)))
    :effect (and (not (at ?p ?x0 ?y))
		 (not (at ?s ?x1 ?y))
		 (not (clear ?x2 ?y))
		 (not (at-goal ?s))
		 (at ?p ?x1 ?y)
		 (at ?s ?x2 ?y)
		 (clear ?x0 ?y))
    )

  (:action push-left
    :parameters (?p - player ?s - stone ?x0 ?x1 ?x2 - xpos ?y - ypos)
    ;; :assoc "L"
    :precondition (and (at ?p ?x0 ?y)
		       (at ?s ?x1 ?y)
		       (clear ?x2 ?y)
		       (dec ?x0 ?x1)
		       (dec ?x1 ?x2)
		       (not (is-goal ?x2 ?y)))
    :effect (and (not (at ?p ?x0 ?y))
		 (not (at ?s ?x1 ?y))
		 (not (clear ?x2 ?y))
		 (not (at-goal ?s))
		 (at ?p ?x1 ?y)
		 (at ?s ?x2 ?y)
		 (clear ?x0 ?y))
    )

  ;; push to goal

  (:action push-up-to-goal
    :parameters (?p - player ?s - stone ?x - xpos ?y0 ?y1 ?y2 - ypos)
    ;; :assoc "U"
    :precondition (and (at ?p ?x ?y0)
		       (at ?s ?x ?y1)
		       (clear ?x ?y2)
		       (inc ?y0 ?y1)
		       (inc ?y1 ?y2)
		       (is-goal ?x ?y2))
    :effect (and (not (at ?p ?x ?y0))
		 (not (at ?s ?x ?y1))
		 (not (clear ?x ?y2))
		 (at ?p ?x ?y1)
		 (at ?s ?x ?y2)
		 (clear ?x ?y0)
		 (at-goal ?s))
    )

  (:action push-down-to-goal
    :parameters (?p - player ?s - stone ?x - xpos ?y0 ?y1 ?y2 - ypos)
    ;; :assoc "D"
    :precondition (and (at ?p ?x ?y0)
		       (at ?s ?x ?y1)
		       (clear ?x ?y2)
		       (dec ?y0 ?y1)
		       (dec ?y1 ?y2)
		       (is-goal ?x ?y2))
    :effect (and (not (at ?p ?x ?y0))
		 (not (at ?s ?x ?y1))
		 (not (clear ?x ?y2))
		 (at ?p ?x ?y1)
		 (at ?s ?x ?y2)
		 (clear ?x ?y0)
		 (at-goal ?s))
    )

  (:action push-right-to-goal
    :parameters (?p - player ?s - stone ?x0 ?x1 ?x2 - xpos ?y - ypos)
    ;; :assoc "R"
    :precondition (and (at ?p ?x0 ?y)
		       (at ?s ?x1 ?y)
		       (clear ?x2 ?y)
		       (inc ?x0 ?x1)
		       (inc ?x1 ?x2)
		       (is-goal ?x2 ?y))
    :effect (and (not (at ?p ?x0 ?y))
		 (not (at ?s ?x1 ?y))
		 (not (clear ?x2 ?y))
		 (at ?p ?x1 ?y)
		 (at ?s ?x2 ?y)
		 (clear ?x0 ?y)
		 (at-goal ?s))
    )

  (:action push-left-to-goal
    :parameters (?p - player ?s - stone ?x0 ?x1 ?x2 - xpos ?y - ypos)
    ;; :assoc "L"
    :precondition (and (at ?p ?x0 ?y)
		       (at ?s ?x1 ?y)
		       (clear ?x2 ?y)
		       (dec ?x0 ?x1)
		       (dec ?x1 ?x2)
		       (is-goal ?x2 ?y))
    :effect (and (not (at ?p ?x0 ?y))
		 (not (at ?s ?x1 ?y))
		 (not (clear ?x2 ?y))
		 (at ?p ?x1 ?y)
		 (at ?s ?x2 ?y)
		 (clear ?x0 ?y)
		 (at-goal ?s))
    )

  ) ;; end of domain definition
